#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "layshell.h"
#include "global.h"
#include "vecttens.h"
#include "intpoints.h"
#include "matrix.h"
#include "vector.h"
#include "tensor.h"
#include "elastisomat.h"
#include "strretalg.h"
#include "alias.h"
#include "element.h"

/**
  This constructor inializes attributes to zero values.
*/
layshell::layshell (void)
{
  nl = 0; nli = 0;
  nm = NULL; idm = NULL;
}


/**
  The destructor
*/
layshell::~layshell (void)
{
  bcup.tm = NULL;
  bcup.idm = NULL;
  bcup.stress = NULL;
  bcup.strain = NULL;
  bcup.other = NULL;
  bcup.eqother = NULL;

  for (long i=0;i<nl;i++)
  {
    delete [] tm[i];
    delete [] idm[i];
  }
  delete [] tm;
  delete [] idm;
  delete [] nm;
}


/**
  This function reads material parameters from the opened text file given
  by the parameter in.

  @param in - pointer to the opened text file

  9/2015   J. Fiedler
*/
void layshell::read (XFILE *in)
{
  long i,j;
//  long eid = Mm->elip[ipp];
  
  xfscanf (in,"%k%ld", "num_lay", &nl);
/*  if (Mc->give_num_lay(eid)!=nl){
    print_err("wrong number of layers defined", __FILE__, __LINE__, __func__);
  }
*/    
  tm   = new mattype*[nl];
  idm  = new long*[nl];
  nm   = new long[nl];
  
  for (i=0;i<nl;i++)
  {
    xfscanf (in,"%ld", &nm[i]);
    tm[i]  = new mattype[nm[i]];
    idm[i] = new long[nm[i]];

    for (j=0;j<nm[i];j++)
    {
      xfscanf(in, "%k%m%k%ld", "mattype", &mattype_kwdset, (int*)(tm[i]+j), "num_inst", idm[i]+j);
      idm[i][j]--;
    }
  }
  
  xfscanf (in,"%k%ld", "num_iter", &nli);
//  xfscanf (in,"%k%lf", "error", &chyba);

}

void layshell::nondiag (matrix &d, long ipp)
{
  long i,j,jj;
  double *th;
  double *zet;
  matrix e(3,3);
  long eid = Mm->elip[ipp];
  double *k, *eps;
  
  th   = Mc->give_layer_thicke(eid);
  zet  = Mc->give_layer_zcoord(eid);
  
  fillm(0,e);
  fillm(0,d);
  backup(ipp,eps,k,0);
  for (i=0;i<nl;i++)
  {
    Mm->ip[ipp].tm  = tm [i];     // mattype aktualni vrstvy
    Mm->ip[ipp].idm = idm[i];     // index sady mat. vl. aktualni sady
    Mm->ip[ipp].nm  = nm [i];
  
    Mm->elmatstiff (e,ipp);
    
    for (j=0;j<3;j++)
      for (jj=0;jj<3;jj++)
        d[j][jj] += zet[i]*th[i]*e[j][jj];
  }
  
  restore_values(ipp,eps,k,0);
}


void layshell::matstiff (matrix &d, long ipp,long ido)
{  
  
  switch (Mm->ip[ipp].ssst){
  case planestress:
  {
    planestiff(d,ipp);
    break;
  }
  case platek:
  {
    platestiff(d,ipp);
    break;
  }
  
  case bar:
  case plbeam:
  case spacebeam:
  case planestrain:
  case planecontact:
  case plates:
  case axisymm:
  case shell:
  case spacestress:
    break;
  }
}

void layshell::planestiff (matrix &d, long ipp)
{
  long i,j,jj;
  double *k, *eps;
  matrix e(3,3);
  double *th;
  long eid = Mm->elip[ipp];

  th   = Mc->give_layer_thicke(eid);

  fillm(0,e);
  fillm(0,d);
  backup(ipp,eps,k,0);
  
  for (i=0;i<nl;i++){
    Mm->ip[ipp].tm  = tm [i];     // mattype aktualni vrstvy
    Mm->ip[ipp].idm = idm[i];     // index sady mat. vl. aktualni sady
    Mm->ip[ipp].nm  = nm [i];
  
    Mm->elmatstiff (e,ipp);
    
    for (j=0;j<3;j++)
      for (jj=0;jj<3;jj++)
        d[j][jj] += th[i]*e[j][jj];
  }
  
  restore_values(ipp,eps,k,0);
}


void layshell::platestiff (matrix &d, long ipp)
{
  long i,j,jj;
  double *k, *eps;
  matrix e(3,3);
  double *th, *zet;
  long eid = Mm->elip[ipp];

  th   = Mc->give_layer_thicke(eid);
  zet  = Mc->give_layer_zcoord(eid);

  fillm(0,e);
  fillm(0,d);
  backup(ipp,eps,k,0);
  
  for (i=0;i<nl;i++){
    Mm->ip[ipp].tm  = tm [i];     // mattype aktualni vrstvy
    Mm->ip[ipp].idm = idm[i];     // index sady mat. vl. aktualni sady
    Mm->ip[ipp].nm  = nm [i];
  
    Mm->elmatstiff (e,ipp);

    for (j=0;j<3;j++)
      for (jj=0;jj<3;jj++)
        d[j][jj] += th[i]*zet[i]*zet[i]*e[j][jj];
  }

  restore_values(ipp,eps,k,0);
}

void layshell::ccnstiff (matrix &c, matrix &cn, matrix &dn, long ipp)
{
  long eid = Mm->elip[ipp];
  double *th, *zet;
  long i,j,jj;
  matrix e(3,3);
  matrix dnm(3,3);
  matrix mat(3,3);
  
  th   = Mc->give_layer_thicke(eid);
  zet  = Mc->give_layer_zcoord(eid);
  
  for (i=0;i<nl;i++){
    Mm->ip[ipp].tm  = tm [i];     // mattype aktualni vrstvy
    Mm->ip[ipp].idm = idm[i];     // index sady mat. vl. aktualni sady
    Mm->ip[ipp].nm  = nm [i];
  
    Mm->elmatstiff (e,ipp);
    
    for (j=0;j<3;j++){
      for (jj=0;jj<3;jj++){
        dn[j][jj] += th[i]*e[j][jj];
      }
    }
    
    for (j=0;j<3;j++){
      for (jj=0;jj<3;jj++){
        dnm[j][jj] += zet[i]*th[i]*e[j][jj];
      }
    }
  }
  
  invm(dn,c,1.0e-20);
  cmulm(-1.0,dnm,mat);
// matrix "c" is used for relation between eps and curavatures
// C = -D^{-1}_{n}D_{nm}
  mxm(c,mat,cn);
  
}


/**


*/
void layshell::nlstresses (long ipp, long im, long ido)
{
  double *k, *eps;
  long i,j;
  vector sig(6);
  vector df(3);
  vector f(3);
  vector deps(3);
  matrix c(3,3),cn(3,3),dn(3,3);
  double normf;

  Mm->ip[ipp].other[1]=1.0;
  k = new double [3];
  eps = new double [3];
  backup(ipp,eps,k,1);
  
  ccnstiff(c,cn,dn,ipp);
  
  for (i=0;i<3;i++)
    for (j=0;j<3;j++)
      f[i]+=dn[i][j]*eps[j];
  
  for (i=0;i<nli;i++)
  {
    compute_internal_forces(sig,ipp,im,ido,eps,k);
    
    for (j=0;j<3;j++)
      df[j]=sig[j]-f[j];
    
    normf = normv(df);
    if (normf<200)
      break;
    
    mxv(c,df,deps);
    for (j=0;j<3;j++)
      eps[j] -= deps[j];
    if (i==nli-1 && normf>200)
      {
	fprintf (stdout,"\n  Iequilibrium has not been reached normdf = %e",normf);
        Mm->ip[ipp].other[1]=0.5;   // reducing the main loading step
      }
  }

  restore_values(ipp,eps,k,1);
  Mm->storestress(0,ipp,sig);
  
  delete [] k;  
  delete [] eps;

}

void layshell::compute_internal_forces (vector &sig, long ipp, long im, long ido, double *eps, double *k)
{
  long i,j;
  long neq;
  double *layth;          // pointer to array of thicnkesses of layers
  double *layz;           // pointer to array of z-coordinates of layers
  long eid = Mm->elip[ipp];
  vector m_int(3);
  vector n_int(3);

  layth = Mc->give_layer_thicke(eid);
  layz  = Mc->give_layer_zcoord(eid);
  
  ido = 2;
  Mm->ip[ipp].other[0]=0.0;

// main loop going through layers
  for (i=0;i<nl;i++)
  {
    Mm->ip[ipp].tm  = tm [i];     // mattype of the current layer
    Mm->ip[ipp].idm = idm[i];     // index of material atributes for the current layer
    Mm->ip[ipp].nm  = nm [i];

    // calculating deformation of the current layer
    for (j=0;j<3;j++)
      Mm->ip[ipp].strain[j] = k[j]*layz[i]+eps[j];

    // calculating stresses of the layer according to the defined material model
    Mm->computenlstresses(ipp,0,ido);

    // adding up the internal forces
    for (j=0;j<3;j++)
    {
      m_int[j] += Mm->ip[ipp].stress[j]*layth[i]*layz[i];
      n_int[j] += Mm->ip[ipp].stress[j]*layth[i];
    }

    // storing layer stresses to the "other" array    
    neq = Mm->givencompeqother(ipp,0);
    for (j=0;j<3;j++)
      Mm->ip[ipp].other[ido+neq+j] = Mm->ip[ipp].stress[j];

    // adding up the gamma parameter
    for (j=0;j<3;j++)
      Mm->ip[ipp].other[0] += Mm->ip[ipp].other[ido+4];

    ido += neq+3;      // recalculation of "ido" for next layer
  }
  
  for (i=0;i<3;i++)
  {
    sig[i]=n_int[i];
    sig[i+3]=m_int[i];
  }
}



/**
  This function evaluates the size of eqother array according to number of layers
  and material models used.

  @param ipp - integration point number in the mechmat ip array.

  @return The function returns the size of eqother array depending of number of layers
          and their material type.

  9/2015 J. Fiedler
*/
long layshell::compeqother(long ipp)
{
  long sizeeqother=0;
  long i;
  mattype *btm = Mm->ip[ipp].tm;
  long bncompstr = Mm->ip[ipp].ncompstr;
  Mm->ip[ipp].ncompstr = 4;

  // loop going through layers
  for (i=0;i<nl;i++){
    Mm->ip[ipp].tm  = tm [i];
    sizeeqother += Mm->givencompeqother(ipp,0)+3;
  }

  Mm->ip[ipp].tm  = btm;  // returning to original "layshell" mattype
  Mm->ip[ipp].ncompstr = bncompstr;
  
  return sizeeqother+2;     // +1 pro soucet gammy,+1 pro ukazatel dstep_red
}


/**
  This function evaluates the size of other array according to number of layers
  and material models used.

  @param ipp - integration point number in the mechmat ip array.

  @return The function returns the size of other array depending of number of layers
          and their material type.

  9/2015 J. Fiedler
*/
long layshell::compother(long ipp)
{
  long sizeother=0;
  long i;
  mattype *btm = Mm->ip[ipp].tm;
  long bncompstr = Mm->ip[ipp].ncompstr;
  Mm->ip[ipp].ncompstr = 4;

  // loop going through layers
  for (i=0;i<nl;i++)
  {
    Mm->ip[ipp].tm  = tm[i];
    sizeother += Mm->givencompother(ipp,0)+3;
  }

  Mm->ip[ipp].tm  = btm;  // returning to original "layshell" mattype
  Mm->ip[ipp].ncompstr = bncompstr;
  
  return sizeother+2;     // +1 pro soucet gammy, +1 pro ukazatel dstep_red
}


/**
  Function backs up some informations stored on integration point and prepares arrays for computation.

  @param ipp - integration point number in the mechmat ip array.

  9/2015 J. Fiedler
*/
void layshell::backup(long ipp,double *eps, double *k, long j)
{
  long i;
  
  bcup.tm   = Mm->ip[ipp].tm;
  bcup.idm  = Mm->ip[ipp].idm;
  bcup.nm   = Mm->ip[ipp].nm;
  bcup.ssst = Mm->ip[ipp].ssst;
  bcup.ncompstr = Mm->ip[ipp].ncompstr;
  Mm->ip[ipp].ssst = planestress;
  Mm->ip[ipp].ncompstr = 4;

  if (j==1)
  {
    bcup.stress = Mm->ip[ipp].stress;
    for (i=0;i<3;i++)
   {
     eps[i]=Mm->ip[ipp].strain[i];
     k[i]=Mm->ip[ipp].strain[i+3];
   }
    Mm->ip[ipp].alloc_strains(1);
    Mm->ip[ipp].alloc_stresses(1);
    Mm->ip[ipp].clean_strains(1);
    Mm->ip[ipp].clean_stresses(1);
  }
}

/**
  Function restores original informations stored on integration point.

  @param ipp - integration point number in the mechmat ip array.

  9/2015 J. Fiedler
*/
void layshell::restore_values(long ipp, double *eps, double *k, long j)
{
  long i;
  
  Mm->ip[ipp].tm       = bcup.tm;
  Mm->ip[ipp].idm      = bcup.idm;
  Mm->ip[ipp].nm       = bcup.nm;
  Mm->ip[ipp].ssst     = bcup.ssst;
  Mm->ip[ipp].ncompstr = bcup.ncompstr;

  if (j==1)
  {
    delete [] Mm->ip[ipp].strain;
    Mm->ip[ipp].alloc_strains(1);
    Mm->ip[ipp].clean_strains(1);
    for (i=0;i<3;i++)
   {
     Mm->ip[ipp].strain[i]=eps[i];
     Mm->ip[ipp].strain[i+3]=k[i];
   }
    delete [] Mm->ip[ipp].stress;
    Mm->ip[ipp].alloc_stresses(1);
    Mm->ip[ipp].clean_stresses(1);
  }
}



/**
  This function updates values in the other array reached in the previous equlibrium state to
  values reached in the new actual equilibrium state.

  @param ipp - integration point number in the mechmat ip array.
  @param im  - index of material type for given ip.
  @param ido - index of internal variables for given material in the ipp other array.

  @return The function does not return anything.
*/
void layshell::updateval (long ipp, long im, long ido)
{
  long i,n = Mm->givencompeqother(ipp,im);

  for (i=0;i<n;i++)
    Mm->ip[ipp].eqother[ido+i]=Mm->ip[ipp].other[ido+i];
}


/**
  This function is used for reducing a step of the main nonlinear loading process.

  @param ipp - integration point number in the mechmat ip array.
  @param im  - index of material type for given ip.
  @param ido - index of internal variables for given material in the ipp other array.

  @return The function returns a ratio of the step reduction.
*/
double layshell::dstep_red(long ipp, long im, long ido)
{
  if (Mm->ip[ipp].other[1]==0.5)
    return (Mm->ip[ipp].other[1]);
  else
    return 1.0;
}

